@@ -11,7 +11,7 @@
 ;;	Marius Vollmer <marius.vollmer@gmail.com>
 ;; Maintainer: Jonas Bernoulli <jonas@bernoul.li>
 
-;; Package-Requires: ((emacs "24.4") (dash "2.12.1") (with-editor "2.5.1"))
+;; Package-Requires: ((emacs "24.4") (dash "2.13.0") (with-editor "2.5.8"))
 ;; Keywords: git tools vc
 ;; Homepage: https://github.com/magit/magit
 
@@ -101,11 +101,10 @@
 ;; files.
 
 ;; Finally this package highlights style errors, like lines that are
-;; too long, or when the second line is not empty.  It may even nag you
-;; when you attempt to finish the commit without having fixed these
-;; issues.  Some people like that nagging, I don't, so you'll have to
-;; enable it.  Which brings me to the last point.  Like any
-;; respectable Emacs package, this one too is highly customizable:
+;; too long, or when the second line is not empty.  It may even nag
+;; you when you attempt to finish the commit without having fixed
+;; these issues.  The style checks and many other settings can easily
+;; be configured:
 ;;
 ;;   M-x customize-group RET git-commit RET
 
@@ -123,6 +122,8 @@
 ;;;; Declarations
 
 (defvar flyspell-generic-check-word-predicate)
+(defvar font-lock-beg)
+(defvar font-lock-end)
 
 (declare-function magit-expand-git-file-name 'magit-git)
 
@@ -176,8 +177,7 @@ The major mode configured here is turned on by the minor mode
   "Hook run at the end of `git-commit-setup'."
   :group 'git-commit
   :type 'hook
-  :options '(
-             git-commit-save-message
+  :options '(git-commit-save-message
              git-commit-setup-changelog-support
              git-commit-turn-on-auto-fill
              git-commit-turn-on-flyspell
@@ -201,17 +201,42 @@ usually honor this wish and return non-nil."
   :type 'hook
   :group 'git-commit)
 
-(defcustom git-commit-summary-max-length 50
-  "Fontify characters beyond this column in summary lines as errors."
+(defcustom git-commit-style-convention-checks '(non-empty-second-line)
+  "List of checks performed by `git-commit-check-style-conventions'.
+Valid members are `non-empty-second-line' and `overlong-summary-line'.
+That function is a member of `git-commit-finish-query-functions'."
+  :options '(non-empty-second-line overlong-summary-line)
+  :type '(list :convert-widget custom-hook-convert-widget)
+  :group 'git-commit)
+
+(defcustom git-commit-summary-max-length 68
+  "Column beyond which characters in the summary lines are highlighted.
+
+The highlighting indicates that the summary is getting too long
+by some standards.  It does in no way imply that going over the
+limit a few characters or in some cases even many characters is
+anything that deserves shaming.  It's just a friendly reminder
+that if you can make the summary shorter, then you might want
+to consider doing so."
   :group 'git-commit
   :safe 'numberp
   :type 'number)
 
-(defcustom git-commit-fill-column 72
-  "Automatically wrap commit message lines beyond this column."
+(defcustom git-commit-fill-column nil
+  "Override `fill-column' in commit message buffers.
+
+If this is non-nil, then it should be an integer.  If that is the
+case and the buffer-local value of `fill-column' is not already
+set by the time `git-commit-turn-on-auto-fill' is called as a
+member of `git-commit-setup-hook', then that function sets the
+buffer-local value of `fill-column' to the value of this option.
+
+This option exists mostly for historic reasons.  If you are not
+already using it, then you probably shouldn't start doing so."
   :group 'git-commit
   :safe 'numberp
-  :type 'number)
+  :type '(choice (const :tag "use regular fill-column")
+                 number))
 
 (defcustom git-commit-known-pseudo-headers
   '("Signed-off-by" "Acked-by" "Cc"
@@ -345,6 +370,8 @@ usually honor this wish and return non-nil."
 (eval-after-load 'recentf
   '(add-to-list 'recentf-exclude git-commit-filename-regexp))
 
+(add-to-list 'with-editor-file-name-history-exclude git-commit-filename-regexp)
+
 (defun git-commit-setup-font-lock-in-buffer ()
   (and buffer-file-name
        (string-match-p git-commit-filename-regexp buffer-file-name)
@@ -374,7 +401,11 @@ usually honor this wish and return non-nil."
     (when (file-accessible-directory-p (file-name-directory it))
       (find-alternate-file it)))
   (when git-commit-major-mode
-    (funcall git-commit-major-mode))
+    (let ((auto-mode-alist (list (cons (concat "\\`"
+                                               (regexp-quote buffer-file-name)
+                                               "\\'")
+                                       git-commit-major-mode))))
+      (normal-mode)))
   (setq with-editor-show-usage nil)
   (with-editor-mode 1)
   (add-hook 'with-editor-finish-query-functions
@@ -415,6 +446,9 @@ usually honor this wish and return non-nil."
   (setq-local comment-end-skip "\n")
   (setq-local comment-use-syntax nil)
   (setq-local font-lock-multiline t)
+  (add-hook 'font-lock-extend-region-functions
+            #'git-commit-extend-region-summary-line
+            t t)
   (font-lock-add-keywords nil (git-commit-mode-font-lock-keywords) t))
 
 (define-minor-mode git-commit-mode
@@ -431,8 +465,12 @@ Don't use it directly, instead enable `global-git-commit-mode'."
 
 (defun git-commit-turn-on-auto-fill ()
   "Unconditionally turn on Auto Fill mode.
-And set `fill-column' to `git-commit-fill-column'."
-  (setq fill-column git-commit-fill-column)
+If `git-commit-fill-column' is non-nil, and `fill-column'
+doesn't already have a buffer-local value, then set that
+to `git-commit-fill-column'."
+  (when (and (numberp git-commit-fill-column)
+             (not (local-variable-p 'fill-column)))
+    (setq fill-column git-commit-fill-column))
   (turn-on-auto-fill))
 
 (defun git-commit-turn-on-flyspell ()
@@ -454,18 +492,23 @@ finally check current non-comment text."
 
 (defun git-commit-check-style-conventions (force)
   "Check for violations of certain basic style conventions.
+
 For each violation ask the user if she wants to proceed anyway.
-This makes sure the summary line isn't too long and that the
-second line is empty."
+Option `git-commit-check-style-conventions' controls which
+conventions are checked."
   (or force
       (save-excursion
         (goto-char (point-min))
         (re-search-forward (git-commit-summary-regexp) nil t)
         (if (equal (match-string 1) "")
             t ; Just try; we don't know whether --allow-empty-message was used.
-          (and (or (equal (match-string 2) "")
+          (and (or (not (memq 'overlong-summary-line
+                              git-commit-style-convention-checks))
+                   (equal (match-string 2) "")
                    (y-or-n-p "Summary line is too long.  Commit anyway? "))
-               (or (not (match-string 3))
+               (or (not (memq 'non-empty-second-line
+                              git-commit-style-convention-checks))
+                   (not (match-string 3))
                    (y-or-n-p "Second line is not empty.  Commit anyway? ")))))))
 
 (defun git-commit-cancel-message ()
@@ -614,6 +657,20 @@ With a numeric prefix ARG, go forward ARG comments."
    ;; Non-empty non-comment second line
    (format "\\(?:\n%s\\|\n\\(.+\\)\\)?" comment-start)))
 
+(defun git-commit-extend-region-summary-line ()
+  "Identify the multiline summary-regexp construct.
+Added to `font-lock-extend-region-functions'."
+  (save-excursion
+    (save-match-data
+      (goto-char (point-min))
+      (when (looking-at (git-commit-summary-regexp))
+        (let ((summary-beg (match-beginning 0))
+              (summary-end (match-end 0)))
+          (when (or (< summary-beg font-lock-beg summary-end)
+                    (< summary-beg font-lock-end summary-end))
+            (setq font-lock-beg (min font-lock-beg summary-beg)
+                  font-lock-end (max font-lock-end summary-end))))))))
+
 (defun git-commit-mode-font-lock-keywords ()
   `(;; Comments
     (,(format "^%s.*" comment-start)
