@@ -94,7 +94,7 @@ and then turned on again when turning off the latter."
      :foreground "black")
     (((class color) (background dark))
      :background "grey25"
-     :foreground "black"))
+     :foreground "white"))
   "Face for blame headings."
   :group 'magit-faces)
 
@@ -197,8 +197,8 @@ and then turned on again when turning off the latter."
   :man-page "git-blame"
   :switches '((?w "Ignore whitespace" "-w")
               (?r "Do not treat root commits as boundaries" "--root"))
-  :options  '((?C "Detect lines moved or copied within a file" "-C")
-              (?M "Detect lines moved or copied between files" "-M"))
+  :options  '((?M "Detect lines moved or copied within a file" "-M")
+              (?C "Detect lines moved or copied between files" "-C"))
   :actions  '((?b "Blame" magit-blame))
   :default-arguments '("-w")
   :default-action 'magit-blame)
@@ -232,60 +232,62 @@ only arguments available from `magit-blame-popup' should be used.
          (if buffer-file-name
              (user-error "Buffer isn't visiting a tracked file")
            (user-error "Buffer isn't visiting a file"))))))
-  (magit-with-toplevel
-    (if revision
-        (magit-find-file revision file)
-      (let ((default-directory default-directory))
+  (let ((toplevel (or (magit-toplevel)
+                      (user-error "Not in git repository"))))
+    (let ((default-directory toplevel))
+      (if revision
+          (magit-find-file revision file)
         (--if-let (find-buffer-visiting file)
             (progn (switch-to-buffer it)
                    (save-buffer))
           (find-file file))))
-    ;; ^ Make sure this doesn't affect the value used below.  b640c6f
-    (widen)
-    (when line
-      (setq magit-blame-recursive-p t)
-      (goto-char (point-min))
-      (forward-line (1- line)))
-    (unless magit-blame-mode
-      (setq magit-blame-cache (make-hash-table :test 'equal))
-      (let ((show-headings magit-blame-show-headings))
-        (magit-blame-mode 1)
-        (setq-local magit-blame-show-headings show-headings))
-      (message "Blaming...")
-      (let ((magit-process-popup-time -1)
-            (inhibit-magit-refresh t))
-        (magit-run-git-async
-         "blame" "--incremental" args
-         "-L" (format "%s,%s"
-                      (line-number-at-pos (window-start))
-                      (line-number-at-pos (1- (window-end nil t))))
-         revision "--" file))
-      (setq magit-blame-process magit-this-process)
-      (set-process-filter magit-this-process 'magit-blame-process-filter)
-      (set-process-sentinel
-       magit-this-process
-       `(lambda (process event)
-          (when (memq (process-status process) '(exit signal))
-            (magit-process-sentinel process event)
-            (magit-blame-assert-buffer process)
-            (with-current-buffer (process-get process 'command-buf)
-              (when magit-blame-mode
-                (let ((magit-process-popup-time -1)
-                      (inhibit-magit-refresh t)
-                      (default-directory ,default-directory))
-                  (magit-run-git-async "blame" "--incremental" ,@args
-                                       ,revision "--" ,file))
-                (setq magit-blame-process magit-this-process)
-                (set-process-filter
-                 magit-this-process 'magit-blame-process-filter)
-                (set-process-sentinel
-                 magit-this-process 'magit-blame-process-sentinel)))))))))
+    (let ((default-directory toplevel))
+      (widen)
+      (when line
+        (setq magit-blame-recursive-p t)
+        (goto-char (point-min))
+        (forward-line (1- line)))
+      (unless magit-blame-mode
+        (setq magit-blame-cache (make-hash-table :test 'equal))
+        (let ((show-headings magit-blame-show-headings))
+          (magit-blame-mode 1)
+          (setq-local magit-blame-show-headings show-headings))
+        (message "Blaming...")
+        (let ((magit-process-popup-time -1)
+              (inhibit-magit-refresh t))
+          (magit-run-git-async
+           "blame" "--incremental" args
+           "-L" (format "%s,%s"
+                        (line-number-at-pos (window-start))
+                        (line-number-at-pos (1- (window-end nil t))))
+           revision "--" file))
+        (setq magit-blame-process magit-this-process)
+        (set-process-filter magit-this-process 'magit-blame-process-filter)
+        (set-process-sentinel
+         magit-this-process
+         `(lambda (process event)
+            (when (memq (process-status process) '(exit signal))
+              (magit-process-sentinel process event)
+              (magit-blame-assert-buffer process)
+              (with-current-buffer (process-get process 'command-buf)
+                (when magit-blame-mode
+                  (let ((magit-process-popup-time -1)
+                        (inhibit-magit-refresh t)
+                        (default-directory ,default-directory))
+                    (magit-run-git-async "blame" "--incremental" ,@args
+                                         ,revision "--" ,file))
+                  (setq magit-blame-process magit-this-process)
+                  (set-process-filter
+                   magit-this-process 'magit-blame-process-filter)
+                  (set-process-sentinel
+                   magit-this-process 'magit-blame-process-sentinel))))))))))
 
 (defun magit-blame-process-sentinel (process event)
   (let ((status (process-status process)))
     (when (memq status '(exit signal))
       (magit-process-sentinel process event)
-      (if (eq status 'exit)
+      (if (and (eq status 'exit)
+               (zerop (process-exit-status process)))
           (message "Blaming...done")
         (magit-blame-assert-buffer process)
         (with-current-buffer (process-get process 'command-buf)
@@ -304,7 +306,8 @@ This is intended for debugging purposes.")
     (setf (process-get process 'partial-line) nil))
   (magit-blame-assert-buffer process)
   (with-current-buffer (process-get process 'command-buf)
-    (when magit-blame-mode
+    (when (and magit-blame-mode
+               (zerop (process-exit-status process)))
       (let ((chunk (process-get process 'chunk))
             (lines (split-string string "\n" t)))
         (unless (string-match-p "\n\\'" string)
@@ -484,9 +487,14 @@ then also kill the buffer."
           (goto-char (or next (point-max))))))))
 
 (defun magit-blame-copy-hash ()
-  "Save hash of the current chunk's commit to the kill ring."
+  "Save hash of the current chunk's commit to the kill ring.
+
+When the region is active, then save that to the `kill-ring',
+like `kill-ring-save' would."
   (interactive)
-  (kill-new (message "%s" (magit-blame-chunk-get :hash))))
+  (if (use-region-p)
+      (copy-region-as-kill nil nil 'region)
+    (kill-new (message "%s" (magit-blame-chunk-get :hash)))))
 
 (defun magit-blame-chunk-get (key &optional pos)
   (--when-let (magit-blame-overlay-at pos)
