@@ -165,13 +165,16 @@ FILE has to be relative to the top directory of the repository."
            (bufA (magit-get-revision-buffer "HEAD" file))
            (bufB (get-buffer (concat file ".~{index}~")))
            (bufBrw (and bufB (with-current-buffer bufB (not buffer-read-only))))
-           (bufC (get-file-buffer file)))
+           (bufC (get-file-buffer file))
+           (fileBufC (or bufC (find-file-noselect file)))
+           (coding-system-for-read
+            (with-current-buffer fileBufC buffer-file-coding-system)))
       (ediff-buffers3
        (or bufA (magit-find-file-noselect "HEAD" file))
        (with-current-buffer (magit-find-file-index-noselect file t)
          (setq buffer-read-only nil)
          (current-buffer))
-       (or bufC (find-file-noselect file))
+       fileBufC
        `((lambda ()
            (setq-local
             ediff-quit-hook
@@ -239,16 +242,12 @@ range)."
        'ediff-revision))))
 
 (defun magit-ediff-compare--read-revisions (&optional arg mbase)
-  (let ((input (or arg (magit-diff-read-range-or-commit "Compare range or commit"
-                                                        nil mbase)))
-        revA revB)
-    (if (string-match magit-range-re input)
-        (progn (setq revA (or (match-string 1 input) "HEAD")
-                     revB (or (match-string 3 input) "HEAD"))
-               (when (string= (match-string 2 input) "...")
-                 (setq revA (magit-git-string "merge-base" revA revB))))
-      (setq revA input))
-    (list revA revB)))
+  (let ((input (or arg (magit-diff-read-range-or-commit
+                        "Compare range or commit"
+                        nil mbase))))
+    (--if-let (magit-split-range input)
+        (-cons-to-list it)
+      (list input nil))))
 
 (defun magit-ediff-read-files (revA revB &optional fileB)
   "Read file in REVB, return it and the corresponding file in REVA.
