@@ -12,7 +12,7 @@
 ;; Author: Jonas Bernoulli <jonas@bernoul.li>
 ;; Maintainer: Jonas Bernoulli <jonas@bernoul.li>
 
-;; Package-Requires: ((emacs "24.4") (async "1.5") (dash "2.12.1"))
+;; Package-Requires: ((emacs "24.4") (async "1.9") (dash "2.13.0"))
 ;; Keywords: bindings
 ;; Homepage: https://github.com/magit/magit
 
@@ -63,14 +63,13 @@
 (declare-function Man-next-section 'man)
 
 ;; For the `:variable' event type.
-(declare-function magit-call-git 'magit-process)
 (declare-function magit-git-string 'magit-git)
 (declare-function magit-refresh 'magit-mode)
+(declare-function magit-get 'magit-git)
 (declare-function magit-set 'magit-git)
 
 ;; For branch actions.
 (declare-function magit-branch-set-face 'magit-git)
-(declare-function magit-local-branch-p 'magit-git)
 
 ;;; Settings
 ;;;; Custom Groups
@@ -107,25 +106,19 @@ One of `man' or `woman'."
   :group 'magit-popup
   :type 'boolean)
 
-(defcustom magit-popup-show-common-commands t
-  "Initially show section with commands common to all popups.
+(defcustom magit-popup-show-common-commands nil
+  "Whether to initially show section with commands common to all popups.
 This section can also be toggled temporarily using \
 \\<magit-popup-mode-map>\\[magit-popup-toggle-show-common-commands]."
+  :package-version '(magit-popup . "2.9.0")
   :group 'magit-popup
   :type 'boolean)
 
-(defcustom magit-popup-use-prefix-argument 'disabled
+(defcustom magit-popup-use-prefix-argument 'default
   "Control how prefix arguments affect infix argument popups.
 
 This option controls the effect that the use of a prefix argument
-before entering a popup has.  The *intended* default is `default',
-but the *actual* default is `disabled'.  This is necessary because
-the old popup implementation did simply forward such a pre-popup
-prefix argument to the action invoked from the popup, and changing
-that without users being aware of it could lead to tears.
-
-`disabled' Bring up a Custom option buffer so that the user reads
-           this and then makes an informed choice.
+before entering a popup has.
 
 `default'  With a prefix argument directly invoke the popup's
            default action (an Emacs command), instead of bringing
@@ -139,8 +132,7 @@ that without users being aware of it could lead to tears.
   :type '(choice
           (const :tag "Call default action instead of showing popup" default)
           (const :tag "Show popup instead of calling default action" popup)
-          (const :tag "Ignore prefix argument" nil)
-          (const :tag "Abort and show usage information" disabled)))
+          (const :tag "Ignore prefix argument" nil)))
 
 ;;;; Custom Faces
 
@@ -180,6 +172,7 @@ that without users being aware of it could lead to tears.
     (define-key map [?\C-x ?\C-s] 'magit-popup-save-default-arguments)
     (define-key map [?\C-g]       'magit-popup-quit)
     (define-key map [??]          'magit-popup-help)
+    (define-key map [?\C-h ?k]    'magit-popup-help)
     (define-key map [?\C-h ?i]    'magit-popup-info)
     (define-key map [?\C-t]       'magit-popup-toggle-show-common-commands)
     (define-key map [?\d]         'backward-button)
@@ -291,6 +284,8 @@ Don't confuse this with `magit-current-popup'.")
 This is intended for internal use only.
 Don't confuse this with `magit-current-popup-args'.")
 
+(defvar-local magit-previous-popup nil)
+
 (defun magit-popup-get (prop)
   "While a popup is active, get the value of PROP."
   (if (memq prop '(:switches :options :variables :actions))
@@ -312,6 +307,9 @@ to determine whether it was invoked from a popup and if so from
 which popup.  If the current command was invoked without the use
 of a popup then this is nil.")
 
+(defvar magit-current-popup-action nil
+  "The popup action now being executed.")
+
 (defvar magit-current-popup-args nil
   "The value of the popup arguments for this editing command.
 
@@ -470,7 +468,7 @@ usually specified in that order):
 
   Members of VALUE may also be nil.  This should only be used
   together with `:max-action-columns' and allows having gaps in
-  the action grit, which can help arranging actions sensibly.
+  the action grid, which can help arranging actions sensibly.
 
 `:default-action'
   The default action of the popup which is used directly instead
@@ -486,10 +484,11 @@ usually specified in that order):
   The value, if specified, should be one of `default' or `popup'.
 
 `:max-action-columns'
-  The maximum number of actions to display on a single line.
-  This helps arranging actions more sensibly.  If there is not
-  enough room to display that many actions on one line, then
-  this is ignored.
+  The maximum number of actions to display on a single line, a
+  number or a function that return a number and takes the name
+  of the section currently being inserted as argument.  If there
+  isn't enough room to display as many columns as specified here,
+  then fewer are used.
 
 `:switches'
   The popup arguments which can be toggled on and off.  VALUE
@@ -506,6 +505,12 @@ usually specified in that order):
   enabled by default) and options with there default values, as
   in \"--OPT=OPTVAL\".
 
+`:variables'
+
+  Git variables which can be set from the popup.  VALUE is a list
+  whose members have the form (KEY DESC COMMAND FORMATTER), see
+  `magit-define-popup-variable' for details.
+
 `:sequence-predicate'
   When this function returns non-nil, then the popup uses
   `:sequence-actions' instead of `:actions', and does not show
@@ -748,10 +753,6 @@ TYPE is one of `:action', `:sequence-action', `:switch', or
          (local   (plist-get def :use-prefix))
          (use-prefix (or local magit-popup-use-prefix-argument)))
     (cond
-     ((and arg (eq magit-popup-use-prefix-argument 'disabled))
-      (customize-option-other-window 'magit-popup-use-prefix-argument)
-      (error (concat "The meaning of prefix arguments has changed.  "
-                     "Please explicitly enable their use again.")))
      ((or (and (eq use-prefix 'default) arg)
           (and (eq use-prefix 'popup) (not arg)))
       (if default
@@ -767,11 +768,15 @@ TYPE is one of `:action', `:sequence-action', `:switch', or
             (call-interactively default))
         (message "%s has no default action; showing popup instead." popup)
         (magit-popup-mode-setup popup mode)))
-     ((memq use-prefix '(disabled default popup nil))
+     ((memq use-prefix '(default popup nil))
       (magit-popup-mode-setup popup mode)
       (when magit-popup-show-help-echo
-        (message (concat "Type C-h i to view popup manual, "
-                         "? to describe an argument or action."))))
+        (message
+         (format
+          "[%s] show common commands, [%s] describe events, [%s] show manual"
+          (propertize "C-t"   'face 'magit-popup-key)
+          (propertize "?"     'face 'magit-popup-key)
+          (propertize "C-h i" 'face 'magit-popup-key)))))
      (local
       (error "Invalid :use-prefix popup property value: %s" use-prefix))
      (t
@@ -806,10 +811,14 @@ TYPE is one of `:action', `:sequence-action', `:switch', or
   (interactive (list last-command-event))
   (let ((action   (magit-popup-lookup event :actions))
         (variable (magit-popup-lookup event :variables)))
+    (when (and variable (not (magit-popup-event-arg variable)))
+      (setq action variable)
+      (setq variable nil))
     (if (or action variable)
-        (let ((magit-current-popup magit-this-popup)
-              (magit-current-popup-args (magit-popup-get-args))
-              (command (magit-popup-event-fun (or action variable))))
+        (let* ((magit-current-popup magit-this-popup)
+               (magit-current-popup-args (magit-popup-get-args))
+               (command (magit-popup-event-fun (or action variable)))
+               (magit-current-popup-action command))
           (when action
             (magit-popup-quit))
           (call-interactively command)
@@ -817,16 +826,17 @@ TYPE is one of `:action', `:sequence-action', `:switch', or
           (unless action
             (magit-refresh-popup-buffer)))
       (if (eq event ?q)
-          (magit-popup-quit)
+          (progn (magit-popup-quit)
+                 (when magit-previous-popup
+                   (magit-popup-mode-setup magit-previous-popup nil)))
         (user-error "%c isn't bound to any action" event)))))
 
 (defun magit-popup-set-variable
     (variable choices &optional default other)
-  (--if-let (--if-let (magit-git-string "config" "--local" variable)
-                (cadr (member it choices))
-              (car choices))
-      (magit-set it variable)
-    (magit-call-git "config" "--unset" variable))
+  (magit-set (--if-let (magit-git-string "config" "--local" variable)
+                 (cadr (member it choices))
+               (car choices))
+             variable)
   (magit-refresh)
   (message "%s %s" variable
            (magit-popup-format-variable-1 variable choices default other)))
@@ -858,9 +868,10 @@ argument is used in which case the popup remains open.
 For a popup named `NAME-popup' that usually means setting the
 value of the custom option `NAME-arguments'."
   (interactive "P")
-  (customize-set-variable (magit-popup-get :variable)
-                          (magit-popup-get-args))
-  (unless arg (magit-popup-quit)))
+  (-if-let (var (magit-popup-get :variable))
+      (progn (customize-set-variable var (magit-popup-get-args))
+             (unless arg (magit-popup-quit)))
+    (user-error "Nothing to set")))
 
 (defun magit-popup-save-default-arguments (arg)
   "Save default value for the arguments for the current popup.
@@ -870,9 +881,10 @@ argument is used in which case the popup remains open.
 For a popup named `NAME-popup' that usually means saving the
 value of the custom option `NAME-arguments'."
   (interactive "P")
-  (customize-save-variable (magit-popup-get :variable)
-                           (magit-popup-get-args))
-  (unless arg (magit-popup-quit)))
+  (-if-let (var (magit-popup-get :variable))
+      (progn (customize-save-variable var (magit-popup-get-args))
+             (unless arg (magit-popup-quit)))
+    (user-error "Nothing to save")))
 
 ;;; Help
 
@@ -954,7 +966,7 @@ and are defined in `magit-popup-mode-map' (which see)."
     (split-window-below)
     (other-window 1)
     (with-no-warnings ; display-buffer-function is obsolete
-      (let ((display-buffer-alist nil)
+      (let ((display-buffer-alist '(("" display-buffer-use-some-window)))
             (display-buffer-function nil))
         (describe-function function)))
     (fit-window-to-buffer)
@@ -1012,6 +1024,7 @@ restored."
                                  val (plist-get def :actions)))))
 
 (defun magit-popup-mode-setup (popup mode)
+  (setq magit-previous-popup magit-current-popup)
   (let ((val (symbol-value (plist-get (symbol-value popup) :variable)))
         (def (symbol-value popup)))
     (magit-popup-mode-display-buffer (get-buffer-create
@@ -1092,6 +1105,8 @@ of events shared by all popups and before point is adjusted.")
         (cl-typecase maxcols
           (keyword (setq maxcols (magit-popup-get maxcols)))
           (symbol  (setq maxcols (symbol-value maxcols)))))
+      (when (functionp maxcols)
+        (setq maxcols (funcall maxcols heading)))
       (when items
         (if (functionp heading)
             (when (setq heading (funcall heading))
@@ -1142,12 +1157,14 @@ of events shared by all popups and before point is adjusted.")
         'type type 'event (magit-popup-event-key ev)))
 
 (defun magit-popup-format-variable-button (type ev)
-  (list (format-spec
-         (button-type-get type 'format)
-         `((?k . ,(propertize (magit-popup-event-keydsc ev)
-                              'face 'magit-popup-key))
-           (?d . ,(funcall (magit-popup-event-arg ev)))))
-        'type type 'event (magit-popup-event-key ev)))
+  (if (not (magit-popup-event-arg ev))
+      (magit-popup-format-action-button 'magit-popup-action-button ev)
+    (list (format-spec
+           (button-type-get type 'format)
+           `((?k . ,(propertize (magit-popup-event-keydsc ev)
+                                'face 'magit-popup-key))
+             (?d . ,(funcall (magit-popup-event-arg ev)))))
+          'type type 'event (magit-popup-event-key ev))))
 
 (defun magit-popup-format-variable
     (variable choices &optional default other width)
@@ -1157,12 +1174,14 @@ of events shared by all popups and before point is adjusted.")
 
 (defun magit-popup-format-variable-1
     (variable choices &optional default other)
+  "Print popup entry for git VARIABLE with possible CHOICES.
+DEFAULT is git's default choice for VARIABLE.  OTHER is a git
+variable whose value may be used as a default."
   (let ((local  (magit-git-string "config" "--local"  variable))
         (global (magit-git-string "config" "--global" variable)))
     (when other
-      (--if-let (magit-git-string "config" other)
-          (setq other (concat other ":" it))
-        (setq other nil)))
+      (setq other (--when-let (magit-get other)
+                    (concat other ":" it))))
     (concat
      (propertize "[" 'face 'magit-popup-disabled-argument)
      (mapconcat
