@@ -45,6 +45,18 @@
 
 ;;; Options
 
+(defcustom magit-git-output-coding-system
+  (and (eq system-type 'windows-nt) 'utf-8)
+  "Coding system for receiving output from Git.
+
+If non-nil, the Git config value `i18n.logOutputEncoding' should
+be set via `magit-git-global-arguments' to value consistent with
+this."
+  :package-version '(magit . "2.9.0")
+  :group 'magit-process
+  :type '(choice (coding-system :tag "Coding system to decode Git output")
+                 (const :tag "Use system default" nil)))
+
 (defcustom magit-process-connection-type (not (eq system-type 'cygwin))
   "Connection type used for the Git process.
 
@@ -55,13 +67,18 @@ If t, use ptys: this enables Magit to prompt for passphrases when needed."
                  (const :tag "pty" t)))
 
 (defcustom magit-need-cygwin-noglob
-  (equal "x0\n" (with-temp-buffer
-                  (let ((process-environment
-                         (append magit-git-environment process-environment)))
-                    (process-file magit-git-executable
-                                  nil (current-buffer) nil
-                                  "-c" "alias.echo=!echo" "echo" "x{0}"))
-                  (buffer-string)))
+  (and (eq system-type 'windows-nt)
+       (with-temp-buffer
+         (let ((process-environment
+                (append magit-git-environment process-environment)))
+           (condition-case e
+               (process-file magit-git-executable
+                             nil (current-buffer) nil
+                             "-c" "alias.echo=!echo" "echo" "x{0}")
+             (file-error
+              (lwarn 'magit-process :warning
+                     "Could not run Git: %S" e))))
+         (equal "x0\n" (buffer-string))))
   "Whether to use a workaround for Cygwin's globbing behavior.
 
 If non-nil, add environment variables to `process-environment' to
@@ -139,10 +156,11 @@ itself from the hook, to avoid further futile attempts."
     ;; match-group 99 is used to identify a host
     "^\\(Enter \\)?[Pp]assword\\( for '\\(?99:.*\\)'\\)?: ?$"
     "^.*'s password: ?$"
-    "^Yubikey for .*: ?$")
+    "^Yubikey for .*: ?$"
+    "^Enter PIN for .*: ?$")
   "List of regexps matching password prompts of Git and its subprocesses.
 Also see `magit-process-find-password-functions'."
-  :package-version '(magit . "2.1.0")
+  :package-version '(magit . "2.8.0")
   :group 'magit-process
   :type '(repeat (regexp)))
 
@@ -227,7 +245,9 @@ optional NODISPLAY is non-nil also display it."
             (when magit-process-log-max
               (magit-process-truncate-log))
           (magit-process-mode)
-          (let ((inhibit-read-only t))
+          (let ((inhibit-read-only t)
+                (magit-insert-section--parent  nil)
+                (magit-insert-section--oldroot nil))
             (make-local-variable 'text-property-default-nonsticky)
             (magit-insert-section (processbuf)
               (insert "\n")))))
@@ -291,8 +311,9 @@ before use.
 Process output goes into a new section in the buffer returned by
 `magit-process-buffer'."
   (run-hooks 'magit-pre-call-git-hook)
-  (apply #'magit-call-process magit-git-executable
-         (magit-process-git-arguments args)))
+  (let ((default-process-coding-system (magit--process-coding-system)))
+    (apply #'magit-call-process magit-git-executable
+           (magit-process-git-arguments args))))
 
 (defun magit-call-process (program &rest args)
   "Call PROGRAM synchronously in a separate process.
@@ -396,9 +417,7 @@ current when this function was called (if it is a Magit buffer
 and still alive), as well as the respective Magit status buffer.
 
 See `magit-start-process' and `with-editor' for more information."
-  (with-editor "GIT_EDITOR"
-    (let ((magit-process-popup-time -1))
-      (magit-run-git-async args))))
+  (magit-with-editor (magit-run-git-async args)))
 
 (defun magit-run-git-sequencer (&rest args)
   "Export GIT_EDITOR and start Git.
@@ -414,9 +433,7 @@ If the sequence stops at a commit, make the section representing
 that commit the current section by moving `point' there.
 
 See `magit-start-process' and `with-editor' for more information."
-  (with-editor "GIT_EDITOR"
-    (let ((magit-process-popup-time -1))
-      (magit-run-git-async args)))
+  (apply #'magit-run-git-with-editor args)
   (set-process-sentinel magit-this-process #'magit-sequencer-process-sentinel)
   magit-this-process)
 
@@ -440,8 +457,9 @@ and still alive), as well as the respective Magit status buffer.
 
 See `magit-start-process' for more information."
   (run-hooks 'magit-pre-start-git-hook)
-  (apply #'magit-start-process magit-git-executable input
-         (magit-process-git-arguments args)))
+  (let ((default-process-coding-system (magit--process-coding-system)))
+    (apply #'magit-start-process magit-git-executable input
+           (magit-process-git-arguments args))))
 
 (defun magit-start-process (program &optional input &rest args)
   "Start PROGRAM, prepare for refresh, and return the process object.
@@ -506,7 +524,8 @@ Magit status buffer."
 
 (defun magit-process-insert-section (pwd program args &optional errcode errlog)
   (let ((inhibit-read-only t)
-        (magit-insert-section--parent magit-root-section))
+        (magit-insert-section--parent magit-root-section)
+        (magit-insert-section--oldroot nil))
     (goto-char (1- (point-max)))
     (magit-insert-section (process)
       (insert (if errcode
@@ -576,19 +595,22 @@ Magit status buffer."
   "Special sentinel used by `magit-run-git-sequencer'."
   (when (memq (process-status process) '(exit signal))
     (magit-process-sentinel process event)
-    (--when-let (magit-mode-get-buffer 'magit-status-mode)
-      (with-current-buffer it
-        (--when-let
-            (magit-get-section
-             `((commit . ,(magit-rev-parse "HEAD"))
-               (,(pcase (car (cadr (-split-at
-                                    (1+ (length magit-git-global-arguments))
-                                    (process-command process))))
-                   ((or "rebase" "am")   'rebase-sequence)
-                   ((or "cherry-pick" "revert") 'sequence)))
-               (status)))
-          (goto-char (magit-section-start it))
-          (magit-section-update-highlight))))))
+    (-when-let (process-buf (process-get process 'process-buf))
+      (when (buffer-live-p process-buf)
+        (-when-let (status-buf (with-current-buffer process-buf
+                                 (magit-mode-get-buffer 'magit-status-mode)))
+          (with-current-buffer status-buf
+            (--when-let
+                (magit-get-section
+                 `((commit . ,(magit-rev-parse "HEAD"))
+                   (,(pcase (car (cadr (-split-at
+                                        (1+ (length magit-git-global-arguments))
+                                        (process-command process))))
+                       ((or "rebase" "am")   'rebase-sequence)
+                       ((or "cherry-pick" "revert") 'sequence)))
+                   (status)))
+              (goto-char (magit-section-start it))
+              (magit-section-update-highlight))))))))
 
 (defun magit-process-filter (proc string)
   "Default filter used by `magit-start-process'."
@@ -683,12 +705,13 @@ Return the matched string suffixed with \": \", if needed."
             (t                             (concat prompt ": "))))))
 
 (defun magit--process-coding-system ()
-  (if magit-process-ensure-unix-line-ending
-      (cons (coding-system-change-eol-conversion
-             (car default-process-coding-system) 'unix)
-            (coding-system-change-eol-conversion
-             (cdr default-process-coding-system) 'unix))
-      default-process-coding-system))
+  (let ((fro (or magit-git-output-coding-system
+                 (car default-process-coding-system)))
+        (to (cdr default-process-coding-system)))
+    (if magit-process-ensure-unix-line-ending
+        (cons (coding-system-change-eol-conversion fro 'unix)
+              (coding-system-change-eol-conversion to 'unix))
+      (cons fro to))))
 
 (defvar magit-credential-hook nil
   "Hook run before Git needs credentials.")
@@ -775,11 +798,11 @@ as argument."
           default-dir (process-get arg 'default-dir)
           section     (process-get arg 'section)
           arg         (process-exit-status arg)))
-  (magit-process-unset-mode-line)
   (when (featurep 'dired)
     (dired-uncache default-dir))
   (when (buffer-live-p process-buf)
     (with-current-buffer process-buf
+      (magit-process-unset-mode-line)
       (let ((inhibit-read-only t)
             (marker (magit-section-start section)))
         (goto-char marker)
@@ -815,8 +838,11 @@ as argument."
                    "Git failed")))
       (if magit-process-raise-error
           (signal 'magit-git-error (list (format "%s (in %s)" msg default-dir)))
-        (--when-let (magit-mode-get-buffer 'magit-status-mode)
-          (setq magit-this-error msg))
+        (when (buffer-live-p process-buf)
+          (with-current-buffer process-buf
+            (-when-let (status-buf (magit-mode-get-buffer 'magit-status-mode))
+              (with-current-buffer status-buf
+                (setq magit-this-error msg)))))
         (message "%s ... [%s buffer %s for details]" msg
                  (-if-let (key (and (buffer-live-p command-buf)
                                     (with-current-buffer command-buf
@@ -847,6 +873,10 @@ as argument."
                              process))))))
 
 ;;; magit-process.el ends soon
+
+(define-obsolete-variable-alias 'magit-log-output-coding-system
+  'magit-git-output-coding-system "Magit 2.9.0")
+
 (provide 'magit-process)
 ;; Local Variables:
 ;; indent-tabs-mode: nil
